/**
 * 
 */
package suicideChess;

/**
 * This class is used for piece representation.
 * 
 * @author Jean-Baptiste H&eacute;tier
 * @version $LastChangedRevision$, $LastChangedDate$  
 */

public class Piece {
	/*===========*
	 * CONSTANTS *
	 *===========*/
	
	
	/*
	 * Take really good care if you want to change those values
	 * Class BitBoard makes intensive use of those
	 */
	public static final int NONE=-1;
	public static final int NO_COLOR = -1;
	//Contants used to detect color
	public static final int WHITE=0;
	public static final int BLACK=1;
	//Constants used in the board representation
	public static final int WHITE_PIECES = WHITE;
	public static final int BLACK_PIECES = BLACK;
	public static final int PAWN = 0;
	public static final int WHITE_PAWN = 2*(PAWN+1) + WHITE;
	public static final int BLACK_PAWN = 2*(PAWN+1) + BLACK;
	public static final int QUEEN = 1;
	public static final int WHITE_QUEEN = 2*(QUEEN+1) + WHITE;
	public static final int BLACK_QUEEN = 2*(QUEEN+1) + BLACK;
	public static final int KING = 2;
	public static final int WHITE_KING = 2*(KING+1) + WHITE;
	public static final int BLACK_KING = 2*(KING+1) + BLACK;
	public static final int BISHOP = 3;
	public static final int WHITE_BISHOP = 2*(BISHOP+1) + WHITE;
	public static final int BLACK_BISHOP = 2*(BISHOP+1) + BLACK;
	public static final int KNIGHT = 4;
	public static final int WHITE_KNIGHT = 2*(KNIGHT+1) + WHITE;
	public static final int BLACK_KNIGHT = 2*(KNIGHT+1) + BLACK;
	public static final int ROOK = 5;
	public static final int WHITE_ROOK = 2*(ROOK+1) + WHITE;
	public static final int BLACK_ROOK = 2*(ROOK+1) + BLACK;
	
	public static final int MAX_PIECE_NUMBER = BLACK_ROOK;
    //offset used for table indexing
    public static final int OFFSET = WHITE_PAWN;
    
	public static final int NB_OF_PIECES = ROOK + 1;
	
	//Constants used for promotion (as used in the xboard protocol)
	public static final char KING_CHAR='k';
	public static final char QUEEN_CHAR='q';
	public static final char BISHOP_CHAR='b';
	public static final char KNIGHT_CHAR='n';
	public static final char ROOK_CHAR='r';
    public static final char PAWN_CHAR='p';

    public static final char WHITE_KING_CHAR='K';
    public static final char WHITE_QUEEN_CHAR='Q';
    public static final char WHITE_BISHOP_CHAR='B';
    public static final char WHITE_KNIGHT_CHAR='N';
    public static final char WHITE_ROOK_CHAR='R';
    public static final char WHITE_PAWN_CHAR='P';
    public static final char BLACK_KING_CHAR='k';
    public static final char BLACK_QUEEN_CHAR='q';
    public static final char BLACK_BISHOP_CHAR='b';
    public static final char BLACK_KNIGHT_CHAR='n';
    public static final char BLACK_ROOK_CHAR='r';
    public static final char BLACK_PAWN_CHAR='p';    
    
	/*==============*
	 * PRIVATE DATA *
	 *==============*/
	
	private int pieceNumber;
	
	/*=============*
	 * CONSTRUCTOR *
	 *=============*/
	
	/**
	 * Construcs a Piece given a piece number. You should use the constants
	 * declared in this class.
	 * @param piece An integer corresponding to a Piece code.
	 */
	public Piece(int piece) {
		pieceNumber = piece;
	}
	
	/**
	 * Construcs a Piece given its color and its standard algebraic name
	 * (declared as constants in this file)
	 * @param piece A character used to represent the Piece.
	 * @param color The color of the piece. (See declared constants)
	 */
	public Piece(char piece, int color) {
		pieceNumber = NONE;
		switch (piece) {
			case PAWN_CHAR:
				pieceNumber = 2*(PAWN+1) + color;
				break;
			case KING_CHAR:
				pieceNumber = 2*(KING+1) + color;
				break;
			case QUEEN_CHAR:
				pieceNumber = 2*(QUEEN+1) + color;
				break;
			case BISHOP_CHAR:
				pieceNumber = 2*(BISHOP+1) + color;
				break;
			case KNIGHT_CHAR:
				pieceNumber = 2*(KNIGHT+1) + color;
				break;
			case ROOK_CHAR:
				pieceNumber = 2*(ROOK+1) + color;
				break;
		}
	}

    /**
     * Construcs a Piece given its standard algebraic name
     * uppercase is white and lowercase is black
     */
    public Piece(char piece) {
        pieceNumber = NONE;
        switch (piece) {
            case WHITE_PAWN_CHAR:
                pieceNumber = 2*(PAWN+1) + WHITE;
                break;
            case WHITE_KING_CHAR:
                pieceNumber = 2*(KING+1) + WHITE;
                break;
            case WHITE_QUEEN_CHAR:
                pieceNumber = 2*(QUEEN+1) + WHITE;
                break;
            case WHITE_BISHOP_CHAR:
                pieceNumber = 2*(BISHOP+1) + WHITE;
                break;
            case WHITE_KNIGHT_CHAR:
                pieceNumber = 2*(KNIGHT+1) + WHITE;
                break;
            case WHITE_ROOK_CHAR:
                pieceNumber = 2*(ROOK+1) + WHITE;
                break;
            case BLACK_PAWN_CHAR:
                pieceNumber = 2*(PAWN+1) + BLACK;
                break;
            case BLACK_KING_CHAR:
                pieceNumber = 2*(KING+1) + BLACK;
                break;
            case BLACK_QUEEN_CHAR:
                pieceNumber = 2*(QUEEN+1) + BLACK;
                break;
            case BLACK_BISHOP_CHAR:
                pieceNumber = 2*(BISHOP+1) + BLACK;
                break;
            case BLACK_KNIGHT_CHAR:
                pieceNumber = 2*(KNIGHT+1) + BLACK;
                break;
            case BLACK_ROOK_CHAR:
                pieceNumber = 2*(ROOK+1) + BLACK;
                break;
        }
    }
    
	/*=========*
	 * METHODS *
	 *=========*/
	
	/**
	 * Returns an integer representing the color of the Piece.
	 * @return An integer.
	 */
	public int getColor() {
		if (pieceNumber!=Piece.NONE) {
			return pieceNumber%2; //cf declaration of BLACK and WHITE above and the pieces above.
		} else {
			return Piece.NO_COLOR;
		}
	}
	
	/**
	 * Returns an integer representing the Piece (Piece.BLACK_PAWN or Piece.WHITE_QUEEN)
	 * @return An integer.
	 */
	public int getPieceNumber() {
		return pieceNumber;
	}
	
	/**
	 * Returns an integer representing the Piece type (Piece.QUEEN or Piece.PAWN)
	 */
	public int getPieceType() {
		return ((pieceNumber/2)-1);
	}
    
    /**
     * Displays the agebraic name of the piece
     * @return String
     */
    public String toString() {
        switch (pieceNumber/2-1) {
        case BISHOP:
            return Character.toString(BISHOP_CHAR);
        case QUEEN:
            return Character.toString(QUEEN_CHAR);
        case KING:
            return Character.toString(KING_CHAR);
        case ROOK:
            return Character.toString(ROOK_CHAR);
        case PAWN:
            return Character.toString(PAWN_CHAR);
        case KNIGHT:
            return Character.toString(KNIGHT_CHAR);
        }
        return "**Warning** in Piece.java";
    }
}
