package suicideChess;

/**
 * This class is used for square representation
 * 
 * @author Jean-Baptiste H&eacute;tier
 * @version $LastChangedRevision$, $LastChangedDate$
 */


public class Square {
	private char file;
	private int fileNb;
	private int rank;

    @SuppressWarnings("serial")
    public class NotAValidSquare extends Exception {
		NotAValidSquare(String s) { super(s); };
	}
	
	/**
	 * Construcs a Square given a String ("a1", "b7", "g5", ...)
	 * @param square A string representing the Square. 
	 */
	
	public Square(String square) throws NotAValidSquare {
		if (square.length()!=2 || !(Character.isLetter(square.toCharArray()[0])) || 
				!(Character.isDigit(square.toCharArray()[1]))) {
			throw new NotAValidSquare(square);	
		}
		file = Character.toLowerCase(square.toCharArray()[0]);
		fileNb = ((int)file) - ((int)'a') + 1;
		rank = Integer.parseInt(square.substring(1,2));
		
		//perform extra check ?
		if (SuicideChess.SQUARE_CHECK_INVALID) {
			if (fileNb<1 || fileNb>Board.NB_OF_FILES) {
				throw new NotAValidSquare(square);
			}
			if (rank<1 || rank>Board.NB_OF_RANKS) {
				throw new NotAValidSquare(square);
			}
		}
	}
    
    /**
     * Constructs a Square by copying another square
     */
    public Square(Square square) {
        this.file = square.file;
        this.fileNb = square.fileNb;
        this.rank = square.rank;
    }
	
	/**
	 * Construcs a Square given a bitboard square (number from 0 to 63)
	 * @param bitboardSquare An integer representing the bitboard square
	 * @see Board
	 */
	public Square(int bitboardSquare) throws NotAValidSquare {
		if ((bitboardSquare < 0) || (bitboardSquare >= Board.NB_OF_SQUARES)) {
			throw new NotAValidSquare(Integer.toString(bitboardSquare));
		}
		fileNb = bitboardSquare%Board.NB_OF_FILES + 1;
		file = (char)(fileNb + ((int)'a') - 1);
		rank = bitboardSquare/Board.NB_OF_FILES + 1;
	}

	/**
	 * Construcs a Square given a rank and a file number
	 * @param file A file number (1 to Board.NB_OF_FILES)
	 * @param rank A rank number (1 to Board.NB_OF_RANKS)
	 */
	public Square(int file, int rank) throws NotAValidSquare {
		if ((file <= 0) || (file > Board.NB_OF_FILES) || (rank <= 0) || (rank > Board.NB_OF_RANKS)) {
			throw new NotAValidSquare("file: "+file+" - rank: "+rank);
		}
		this.fileNb = file;
		this.file = (char)(this.fileNb + ((int)'a') - 1);
		this.rank = rank;
	}

	
	/**
	 * Returns the file of a Square.
	 * @return char representing the file
	 */
	
	public char getFile() {
		return file;
	}
	
	/**
	 * Returns the number of the Rank
	 * @return int representing the rank
	 */
	public int getRank() {
		return rank;
	}
	
	/**
	 * Returns the number of the File
	 * @return int representing the file
	 */
	public int getFileNb() {
		return fileNb;
	}
	
    /**
     * Returns a boolean telling if two squares are the same
     * @param that Square to compare to.
     * @return boolean
     */
    public boolean isEqual(Square that) {
        return ((this.fileNb==that.fileNb)&&(this.rank==that.rank));
    }
    
    
	/**
	 * Converts a Square back to a String
	 * @return String
	 */
	public String toString() {
		return String.valueOf(file)+String.valueOf(rank);
	}
}
