package suicideChess;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * This class is used to load problems from a file
 * 
 * @author Jean-Baptiste H&eacute;tier
 * @version $LastChangedRevision$, $LastChangedDate$
 * 
 */
public class SuicideProblems {
    
    @SuppressWarnings("serial")
    public static class NoSuchSuicideProblem extends Exception {
        NoSuchSuicideProblem(String s) { super(s); };
    }
    
    //This array can hold setup for different problems.
    private static String[] suicideProblems;
    static void load() { suicideProblemsLoad("/problems"); } //initialise with file 'problems' if found in current directory
    
    /**
     * How many problems are loaded
     */
    public static int numberOfProblems() {
        if (suicideProblems[0].equals("")) {
            return 0;
        } else {
            return suicideProblems.length;
        }
    }
    
    /**
     * Return FEN notation string for given problem number
     * @param problemNb
     * @throws NoSuchSuicideProblem 
     */
    public static String getProblemNumber(int problemNb) throws NoSuchSuicideProblem {
        if ((problemNb > numberOfProblems())||(problemNb<1)) {
            throw new NoSuchSuicideProblem("*** Invalid Entry: "+problemNb+" ***");
        }
        return suicideProblems[problemNb-1];
    }
    
    /**
     * This function is used to load other problem files
     * @param file
     */
    public static void suicideProblemsLoad(String file) {
        StringBuffer problemBuffer = new StringBuffer();
        //declared here only to make visible to finally clause
        BufferedReader problemReader = null;
        try {
            if(file.equals("/problems")) {
                problemReader = new BufferedReader(new InputStreamReader(SuicideProblems.class.getResourceAsStream(file)));
            } else {
                problemReader = new BufferedReader(new FileReader(file));
            }
            String line = null; //not declared within while loop
            while ((line = problemReader.readLine()) != null) {
                if (!line.startsWith("#")) { //ignore lines starting with # (comments)
                    problemBuffer.append(line);
                    problemBuffer.append("\n"); //I will then use this symbol to break the buffer in an array
                }
            }
        }
        catch (FileNotFoundException e) {
            System.out.println("File '"+file+"' not found. Problems won't be available.");
        }
        catch (IOException e){
            System.out.println("Error reading file '"+file+"'.");
        }
        finally {
            try {
                if (problemReader!= null) {
                    problemReader.close();
                }
            }
            catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        suicideProblems = problemBuffer.toString().split("\n");
        System.out.println(numberOfProblems()+" problem studies loaded.");
    }


}
