/**
 * 
 */
package suicideChess;

import java.io.IOException;


/**
 * Class used to communicate with XBoard.
 * @author Jean-Baptiste H&eacute;tier
 * @version $LastChangedRevision$, $LastChangedDate$
 */
public class XBoardProtocol {
    
    //Constants
    /**
     * Received 'xboard' command
     */
    public static final int XBOARD = 0;
    /**
     * Received a 'protover' >= 2
     */
    public static final int PROTOVER = 1;
    /**
     * Received a 'protover' < 2
     */
    public static final int NOPROTOVER = 2;
    /**
     * Reveived command 'new'
     */
    public static final int NEW = 3;
    /**
     * Reveiced command 'quit'
     */
    public static final int QUIT = 4;
    /**
     * Received a move
     */ 
    public static final int MOVE = 5;
    /**
     * Received an invalid move
     */
    public static final int INVALID = 6;
    /**
     * XBoard tells the computer to play the current color
     */
    public static final int GO = 7;
    /**
     * XBoard user asks for a Hint
     */
    public static final int HINT = 8;
    /**
     * XBoard asks for computer to be put in force mode.
     */
    public static final int FORCE = 9;
    /**
     * XBoard did not accept sending moves with usermove
     */
    public static final int ACCEPTED_USERMOVE = 10;
    /**
     * XBoard did not accept variant suicide chess
     */
    public static final int NOT_ACCEPTED_SUICIDE = 11;
    /**
     * XBoard sent a ping signal
     */
    public static final int PING = 12;
    /**
     * XBoard sent an Undo signal
     */
    public static final int UNDO = 13;
    /**
     * Xboard sent a remove signal (treated at two undos)
     */
    public static final int REMOVE = 14;
    /**
     * Xboard asks to display thinking output
     */
    public static final int POST = 15;
    /**
     * Xboard asks NOT to display thinking output
     */
    public static final int NOPOST = 16;
    /**
     * XBoard changes the board position
     */
    public static final int SETBOARD = 17;
    /**
     * XBoard sends a signal to limit ply depth
     */
    public static final int SETPLY = 18;
    /**
     * XBoard sends an accepted signal
     */
    public static final int ACCEPTED = 19;
    /**
     * XBoard sends an 'variant suicide' signal
     */
    public static final int VARIANT_SUICIDE = 20;
    /**
     * XBoard sends a book request
     */
    public static final int BOOK = 21;
    /**
     * Unknown command
     */
    public static final int UNKNOWN = -1;
    
    /**
     * This function initialises the communications with XBoard.
     * @throws IOException If SuicideChess.XBOARDPROTOCOL_CREATES_LOGFILE is true then the initialisation
     * will create a logfile.   
     */
    public static void initialise() throws IOException {
        //done=1 is here to tell that the program has finish requesting features
        System.out.println("feature usermove=1"); //sends moves like 'usermove e2e4'
        System.out.println("feature myname=\""+SuicideChess.NAME+"\"");
        System.out.println("feature sigint=0 sigterm=0"); //do not interrupt me
        System.out.println("feature variants=\"suicide\"");
        System.out.println("feature ping=1 setboard=1");
        System.out.println("feature time=0 draw=0 reuse=0 analyze=0 colors=0");
        System.out.println("feature done=1");
        if (SuicideChess.playInACSII()) 
            System.out.println();
    }
    

    /**
     * Sends a move message to XBoard
     * @param move The {@link Move} to be made
     * @see Move
     */
    public static void doMove(Move move) {
        System.out.println("move "+move.toString());
    }
    
    /**
     * Interprets XBoard Commands
     * @param command The command to be interpreted
     * @return a integer reprenting the command (se constants defined in XBoardProtocol.java
     */
    
    public static int getCommand(String command) {
        if (command.equals("xboard")) {
            return XBOARD;
        } else if (command.startsWith("protover")) {
            if (Integer.parseInt(command.substring(9))>=2)
                return PROTOVER;
            return NOPROTOVER;
        } else if (command.equals("new")) {
            return NEW;
        } else if (command.equals("quit")) {
            return QUIT;
        } else if (command.startsWith("usermove")) {
            return MOVE;
        } else if (command.equals("rejected variants")) {
            return NOT_ACCEPTED_SUICIDE;
        } else if (command.equals("accepted usermove")) {
            return ACCEPTED_USERMOVE;
        } else if (command.equals("go")) {
            return GO;
        } else if (command.equals("hint")) {
            return HINT;
        } else if (command.equals("force")) {
            return FORCE;
        } else if (command.startsWith("ping")) {
            return PING;
        } else if (command.equals("undo")) {
            return UNDO;
        } else if (command.equals("remove")) {
            return REMOVE;
        } else if (command.equals("post")) {
            return POST;
        } else if (command.equals("nopost")) {
            return NOPOST;
        } else if (command.startsWith("setboard")) {
            return SETBOARD;
        } else if (command.startsWith("sd")) {
            return SETPLY;
        } else if (command.startsWith("accepted")) {
            return ACCEPTED;
        } else if (command.equals("variant suicide")) {
            return VARIANT_SUICIDE;
        } else if (command.equals("bk")) {
            return BOOK;
        }

        return UNKNOWN;
    }
}
